from django.conf.urls import url
from django.views.i18n import set_language
from touchtechnology.common.views import set_timezone

urlpatterns = [
    url(r'^i18n/set-language$', set_language, name='set-language'),
    url(r'^i18n/set-timezone$', set_timezone, name='set-timezone'),
]



from django.db import transaction
from django.test.utils import override_settings
from test_plus import TestCase
from touchtechnology.news.tests import factories


@override_settings(ROOT_URLCONF='example_app.urls')
class InvalidDateTest(TestCase):

    def test_archive_day(self):
        self.get('news:article', year='2013', month='feb', day='31')
        self.response_404()

    def test_article(self):
        self.get('news:article', year='2013', month='feb', day='31',
                 slug='tfms-new-generaltechnical-manager')
        self.response_404()


@override_settings(ROOT_URLCONF='example_app.urls')
class FeedTest(TestCase):

    def test_atom(self):
        self.assertGoodView('news:feed', format='atom')

    def test_rss(self):
        self.assertGoodView('news:feed', format='rss')


@override_settings(ROOT_URLCONF='example_app.urls')
class SiteTest(TestCase):

    def assertGoodArticleView(self, article):
        self.assertGoodView(
            'news:article',
            year=article.published.year,
            month=article.published.strftime('%b').lower(),
            day=article.published.day,
            slug=article.slug)

    def test_article(self):
        article = factories.ArticleFactory.create(
            headline='This is a predictable headline!')
        self.assertEqual(article.slug, 'this-is-a-predictable-headline')
        self.assertGoodArticleView(article)

    def test_article_one_category(self):
        categories = factories.CategoryFactory.create_batch(1)
        article = factories.ArticleFactory.create()
        article.categories.set(categories)
        self.assertGoodArticleView(article)

    def test_article_many_categories(self):
        categories = factories.CategoryFactory.create_batch(3)
        article = factories.ArticleFactory.create()
        article.categories.set(categories)
        self.assertGoodArticleView(article)

    def test_multiple_articles_related_categories(self):
        with transaction.atomic():
            categories = factories.CategoryFactory.create_batch(3)
            articles = factories.ArticleFactory.create_batch(2)
            for article in articles:
                article.categories.set(categories)
        self.assertGoodArticleView(article)



import pickle

from django.test import TestCase
from touchtechnology.common.sites import AccountsSite


class AppCacheTests(TestCase):

    def test_account_site_simple(self):
        site = AccountsSite()
        pickle.loads(pickle.dumps(site))



from dateutil.parser import parse as parse_datetime
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.utils.functional import wraps
from django.views.decorators.http import last_modified
from touchtechnology.common.models import SitemapNode
from touchtechnology.news.models import Article, Category


@method_decorator
def date_view(f, *a, **kw):
    @wraps(f)
    # @last_modified
    def _decorated(*args, **kwargs):
        year = kwargs.pop('year')
        month = kwargs.pop('month', None)
        day = kwargs.pop('day', None)
        try:
            datetime = parse_datetime('{0}-{1}-{2}'.format(
                year, month or 1, day or 1))
        except ValueError:
            raise Http404

        if settings.USE_TZ:
            datetime = timezone.make_aware(datetime, timezone.utc)

        kwargs['date'] = datetime

        # So we can run an optimal query for finding the last modified time for
        # a view in other decorators, pass in the "delta" that should be used
        if day is None:
            kwargs['delta'] = 'months'
        if month is None:
            kwargs['delta'] = 'years'

        return f(*args, **kwargs)
    return _decorated


@method_decorator
@last_modified
def news_last_modified(request, **kwargs):
    last_modified_datetimes = []
    try:
        last_modified_datetimes.append(
            Article.objects.live().latest('last_modified').last_modified)
    except ObjectDoesNotExist:
        pass
    try:
        last_modified_datetimes.append(
            Category.objects.latest('last_modified').last_modified)
    except ObjectDoesNotExist:
        pass
    try:
        last_modified_datetimes.append(
            SitemapNode.objects.latest('last_modified').last_modified)
    except ObjectDoesNotExist:
        pass
    return max(last_modified_datetimes)


@method_decorator
@last_modified
def last_modified_article(request, **kwargs):
    queryset = Article.objects.live()

    date = kwargs.get('date')
    if date is not None:
        delta = {kwargs.get('delta', 'days'): 1}
        date_range = (date, date + relativedelta(**delta))
        queryset = queryset.filter(published__range=date_range)

    slug = kwargs.get('slug')
    if slug is not None:
        queryset = queryset.filter(slug=slug)

    if not queryset:
        return
    return queryset.latest('last_modified').last_modified



from __future__ import unicode_literals

import datetime
import itertools
import logging
import re
from collections import defaultdict
from decimal import Decimal

import six
from dateutil.rrule import DAILY, WEEKLY, rrule, rruleset
from django.conf import settings
from django.db.models import Max
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from first import first
from tournamentcontrol.competition.models import Match, Stage, StageGroup, Team, UndecidedTeam
from tournamentcontrol.competition.utils import (
    ceiling, final_series_rounds, grouper, round_robin_format, single_elimination_final_format,
)

win_lose_re = re.compile(r'(?P<result>[WL])(?P<match_id>\d+)')

logger = logging.getLogger(__name__)


def coerce_datetime(start_date):
    """
    When operating with time zone aware objects we're better off converting
    our plain old date instances to datetime instances before handing them
    to the rruleset.

    If USE_TZ is True and the datetime isn't aware (it may have been passed
    in as an aware datetime) then also set the time zone to UTC.
    """
    if isinstance(start_date, datetime.date):
        start_date = datetime.datetime.combine(start_date, datetime.time())
    if settings.USE_TZ:
        if not timezone.is_aware(start_date):
            start_date = timezone.make_aware(
                start_date, timezone.get_current_timezone())
    return start_date


def weekly_date_generator(stage, start_date=None):
    if start_date is None:
        start_date = stage.division.season.start_date
    start_date = coerce_datetime(start_date)
    logger.debug("Weekly date generator: '%s' (%s)", stage.title, start_date)
    ruleset = rruleset()
    ruleset.rrule(rrule(WEEKLY, dtstart=start_date))
    for exclusion in stage.division.season.exclusions.dates('date', 'day'):
        exclusion = coerce_datetime(exclusion)
        ruleset.exdate(exclusion)
        logger.debug("EXCLUSION (%s) %s",
                     stage.division.season.title, exclusion)
    for exclusion in stage.division.exclusions.dates('date', 'day'):
        exclusion = coerce_datetime(exclusion)
        ruleset.exdate(exclusion)
        logger.debug("EXCLUSION (%s) %s", stage.division.title, exclusion)
    return iter(ruleset)


def tournament_date_generator(stage, start_date=None, games_per_day=None):
    if start_date is None:
        start_date = stage.division.season.start_date
    start_date = coerce_datetime(start_date)

    if games_per_day is None:
        # in case `games_per_day` is not set on the division, fallback to 1
        games_per_day = stage.division.games_per_day or 1

    logger.debug("Tournament date generator: '%s' (%s:%s)",
                 stage.title, start_date, games_per_day)

    ruleset = rruleset()
    ruleset.rrule(rrule(DAILY, dtstart=start_date))

    for exclusion in stage.division.season.exclusions.dates('date', 'day'):
        exclusion = coerce_datetime(exclusion)
        ruleset.exdate(exclusion)
        logger.debug("EXCLUSION (%s) %s",
                     stage.division.season.title, exclusion)

    for exclusion in stage.division.exclusions.dates('date', 'day'):
        exclusion = coerce_datetime(exclusion)
        ruleset.exdate(exclusion)
        logger.debug("EXCLUSION (%s) %s", stage.division.title, exclusion)

    for dates in itertools.imap(
            lambda dt: itertools.repeat(dt.date(), games_per_day), ruleset):
        for date in dates:
            yield date


def optimum_tournament_pool_count(number_of_teams, days_available,
                                  max_per_day=1, min_per_day=1):
    """
    Using the available input variables, decide how many pools to divide a
    division into.

    :param number_of_teams: number of teams that make up the division
    :type number_of_teams: int
    :param days_available: number of days available for play
    :type days_available: int
    :param max_per_day: max number of games a team may play in one day
    :type max_per_day: int
    :param min_per_day: min number of prelim games a team must play in one day
    :type min_per_day: int
    :return: number of pools to split the division into
    :rtype: decimal.Decimal or None
    """
    # turn all of our input into Decimal types
    number_of_teams = Decimal(number_of_teams)
    days_available = Decimal(days_available)
    max_per_day = Decimal(max_per_day)
    min_per_day = Decimal(min_per_day)

    for n in range(4):
        number_of_pools = pow(2, n)
        largest_pool_size = ceiling(number_of_teams / number_of_pools)
        # smallest_pool_size = floor(number_of_teams / number_of_pools)
        preliminary_rounds = largest_pool_size - 1
        elimination_rounds = final_series_rounds(number_of_pools)
        elimination_days = ceiling(elimination_rounds / max_per_day, 0.5)
        total_rounds = preliminary_rounds + elimination_rounds
        days_required = total_rounds / max_per_day

        # Make sure we are playing enough games per day through preliminary
        # stage of the tournament.
        if preliminary_rounds / max_per_day > \
                days_available - elimination_days:
            logger.warn("Too many games (%s) to be played before the finals "
                        "with %s pools.", preliminary_rounds, number_of_pools)
            continue

        # Make sure we are playing enough games during the preliminary stages
        if preliminary_rounds / max_per_day > \
                days_available - elimination_days:
            logger.warn("Not enough time (%s days) to play %s games at %s "
                        "games per day.", days_available - elimination_days,
                        preliminary_rounds, max_per_day)
            continue

        if days_required <= days_available:
            return number_of_pools


def seeded_tournament(seeded_team_list, days_available, max_per_day=1,
                      min_per_day=1):
    """
    Using the available input variables, divide the list of seeded teams into
    the appropriate number of pools. Produce suitable draw formats definitions
    to execute the tournament.

    :param seeded_team_list: list of teams ordered by seeding rank
    :type seeded_team_list: int
    :param days_available: number of days available for play
    :type days_available: int
    :param max_per_day: max number of games a team may play in one day
    :type max_per_day: int
    :param min_per_day: min number of prelim games a team must play in one day
    :type min_per_day: int
    :returns: pools (list of lists) and draw_formats (list of dicts)
    :rtype: dict
    """
    number_of_teams = len(seeded_team_list)
    number_of_pools = optimum_tournament_pool_count(
        number_of_teams,
        days_available,
        max_per_day,
        min_per_day,
    )

    if isinstance(first(seeded_team_list), six.string_types):
        @python_2_unicode_compatible
        class Team(object):
            def __init__(self, st, order):
                self.st = st
                self.order = order

            def __cmp__(self, other):
                return cmp(self.order, other.order)

            def __str__(self):
                return self.st
        seeded_team_list = [
            Team(t, order=order)
            for order, t in enumerate(seeded_team_list, 1)
        ]

    if number_of_pools is None:
        raise ValueError("Incompatible set of constraints")

    # split teams into number of pools, employing the "serpent" pattern
    pools = sorted(zip(*[
        g if i % 2 else reversed(g)
        for i, g in enumerate(grouper(seeded_team_list, number_of_pools))]),
                   key=None)

    # remove any None items from each pool
    pools = [filter(None, pool) for pool in pools]

    # produce round robin formats
    draw_formats = [
        {
            'label': 'Round Robin (%d/%d teams)' % (size - 1, size),
            'format': round_robin_format(range(1, size + 1)),
        }
        # unique set of pool sizes requiring individual draw formats
        for size in sorted(set([
            ceiling(size, 2)
            for size in [len(pool) for pool in pools]
        ]))
    ]

    # produce finals formats
    draw_formats += [
        {
            'label': _("Final Series (%s pools)") % number_of_pools,
            'format': single_elimination_final_format(
                number_of_pools,
                bronze_playoff=_("Bronze Medal"),
            ),
        }
    ]

    return dict(pools=pools, draw_formats=draw_formats)


@python_2_unicode_compatible
class RoundDescriptor(object):
    def __init__(self, count, round_label, **kwargs):
        self.count = count
        self.round_label = round_label
        self.matches = []

    def add(self, match):
        self.matches.append(match)

    def generate(self, generator, stage, date):
        return map(
            lambda m: m.generate(generator, stage, self, date), self.matches)

    def __str__(self):
        return '\n'.join(
            ['ROUND %s' % self.round_label] + map(unicode, self.matches))


@python_2_unicode_compatible
class MatchDescriptor(object):
    def __init__(self, match_id, home_team, away_team, match_label=None,
                 **kwargs):
        self.match_id = match_id
        self.home_team = home_team
        self.away_team = away_team
        self.match_label = match_label

    def __str__(self):
        if self.match_label:
            return '%s: %s vs %s "%s"' % (
                self.match_id, self.home_team,
                self.away_team, self.match_label)
        return '%s: %s vs %s' % (
            self.match_id, self.home_team, self.away_team)

    def generate(self, generator, stage, round, date):
        if isinstance(stage, StageGroup):
            stages = {
                'stage': stage.stage,
                'stage_group': stage,
            }
            include_in_ladder = stage.stage.keep_ladder
        else:
            stages = {'stage': stage}
            include_in_ladder = stage.keep_ladder

        match = Match(
            label=self.match_label or round.round_label,
            include_in_ladder=include_in_ladder,
            round=round.count,
            date=date,
            **stages)

        setattr(match, 'descriptor', self)

        home_team = generator.team(self.home_team)
        away_team = generator.team(self.away_team)

        if home_team is None:
            match.is_bye = True
        elif isinstance(home_team, UndecidedTeam):
            match.home_team_undecided = home_team
        elif isinstance(home_team, Team):
            match.home_team = home_team
        else:
            match.home_team_eval = home_team

        if away_team is None:
            match.is_bye = True
        elif isinstance(away_team, UndecidedTeam):
            match.away_team_undecided = away_team
        elif isinstance(away_team, Team):
            match.away_team = away_team
        else:
            match.away_team_eval = away_team

        if match.home_team_eval or match.away_team_eval:
            match.evaluated = False

        return (self.match_id, match)


class MatchCollection(object):
    def __init__(self):
        self.keyed = {}
        self.iterable = []
        # QuerySet faking attributes
        self.ordered = True
        # self.db = 'default'

    def __add__(self, other):
        for key, value in other.keyed.items():
            self.keyed[key] = value
        self.iterable += other.iterable
        self.iterable.sort(key=lambda m: m.date)
        return self

    def __getitem__(self, key):
        try:
            return self.keyed[key]
        except KeyError:
            return self.iterable[key]

    def __iter__(self, *args, **kwargs):
        return iter(self.iterable)

    def __len__(self):
        return len(self.iterable)

    def __repr__(self):
        return repr(self.iterable)

    def add(self, match_id, match):
        self.keyed.setdefault(match_id, []).append(match)
        self.iterable.append(match)

    def get(self, match_id):
        return self.keyed.get(match_id)

    def get_latest(self, match_id, **kwargs):
        return self.get(match_id)[-1]

    def save(self):
        for match in self.iterable:
            # I'm unsure if this is a Django related bug, but when I think it
            # might related to a database optimisation - see
            # djangoproject:#6886 - to prevent a hit when you've just assigned.
            #
            # To fix our case, simply re-assign the cached object back so that
            # we update the id field before our database INSERT.
            if match.home_team_eval_related:
                match.home_team_eval_related = match.home_team_eval_related
            if match.away_team_eval_related:
                match.away_team_eval_related = match.away_team_eval_related
            match.save()


class DrawGenerator(object):

    regex = re.compile(
        r'''  # noqa
        (?:
            (?P<round>ROUND(?:\ +(?P<round_label>\d+|[\S ]+))?)         # Start a new Round section
        )

        |                                                               # OR

        (?:
            (?P<match_id>\d+):\s*                                       # match ID (with whitespace)
            (?P<home_team>(?:(?:S\d+)?(?:(?:G\d+)?P|[WL])|[PWL])?\d+)   # home team ID
            \s*vs\s*                                                    # versus literal (with whitespace)
            (?P<away_team>(?:(?:S\d+)?(?:(?:G\d+)?P|[WL])|[PWL])?\d+)   # away team ID
            \ *(?P<match_label>[\S ]+)?                                 # match label (optional)
        )
        ''',
        re.VERBOSE,
    )

    def __init__(self, stage, start_date=None):
        self.rounds = []
        self.stage = stage
        self.start_date = start_date
        self.teams = defaultdict(lambda: None)
        if isinstance(stage, Stage):
            if stage.order > 1:
                queryset = stage.undecided_teams.all()
            else:
                queryset = stage.division.teams.all()
            teams = dict(enumerate(queryset))
        elif isinstance(stage, StageGroup):
            if stage.stage.order > 1:
                queryset = stage.undecided_teams.all()
            else:
                queryset = stage.teams.all()
            teams = dict(enumerate(queryset))
        self.teams.update(teams)

    def team(self, text):
        if text.isdigit():
            return self.teams[int(text) - 1]
        return text

    def parse(self, text):
        round = None
        count = 1

        for line in self.regex.finditer(text):
            data = line.groupdict()

            if data.get('round'):
                round = RoundDescriptor(count=count, **data)
                self.rounds.append(round)
                count += 1
                continue

            match = MatchDescriptor(**data)
            round.add(match)

    def generate(self, n=None, offset=0):
        # if n is not specified generate one complete round
        if n is None:
            n = len(self.rounds)

        rounds = itertools.cycle(self.rounds)

        if self.stage.division.season.mode == WEEKLY:
            logger.debug("WEEKLY generator mode: '%s' (%s)",
                         self.stage.title, self.start_date)
            dates = weekly_date_generator(self.stage, self.start_date)
        elif self.stage.division.season.mode == DAILY:
            logger.debug("DAILY generator mode: '%s' (%s)",
                         self.stage.title, self.start_date)
            dates = tournament_date_generator(self.stage, self.start_date)
        else:
            logger.debug("Fallback date generator [%s]: '%s' (%s)",
                         self.stage.division.season.mode, self.stage.title,
                         self.start_date)
            dates = itertools.cycle([None])

        matches = MatchCollection()

        # Work out what stage we come after, could be with reference to a
        # Stage or a StageGroup so introspect the type.
        try:
            initial = 1
            if isinstance(self.stage, Stage):
                follows = self.stage.comes_after
            elif isinstance(self.stage, StageGroup):
                follows = self.stage.stage.comes_after
        except Stage.DoesNotExist:
            pass
        else:
            # Set our initial round number, if we're following another stage,
            # keep the number ticking over from that point.
            if follows is not None:
                initial += follows.matches.aggregate(
                    max=Max('round')).get('max') or 0

        for i in range(offset + initial, offset + n + initial):
            date = next(dates)
            round = next(rounds)
            for match_id, match in round.generate(self, self.stage, date):
                match.round = i
                matches.add(match_id, match)

                # update the related match values and store only the W or L
                for team in ('home', 'away'):
                    team_eval = win_lose_re.match(
                        getattr(match, team + '_team_eval') or '')
                    if team_eval:
                        decode = team_eval.groupdict()
                        setattr(match, team + '_team_eval_related',
                                matches.get_latest(**decode))
                        setattr(match, team + '_team_eval',
                                decode.get('result'))

        return matches

    @classmethod
    def validate(cls, text):
        errors = set()
        keys = set()
        for line, data in enumerate(text.splitlines()):
            match = cls.regex.match(data)
            if not match:
                errors.add(line)
                continue
            match_id = match.groupdict().get('match_id')
            if match_id:
                if match_id in keys:
                    errors.add(line)
                keys.add(match_id)
        if errors:
            raise ValueError("Draw formula is invalid: line(s) '%s' are not "
                             "in the correct format." %
                             ", ".join(map(str, errors)))
        return True



from __future__ import unicode_literals

from operator import itemgetter

import pytz
from django import forms
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


def timezone_choice(tzname, country=None):
    """
    Given a timezone string, convert it to a two-tuple suitable
    for a choice in a FormField.

    >>> timezone_choice('UTC')
    ('UTC', 'UTC')
    >>> timezone_choice('Australia/Sydney')
    ('Australia/Sydney', 'Sydney')
    >>> timezone_choice('America/Indiana/Indianapolis')
    ('America/Indiana/Indianapolis', 'Indianapolis (Indiana)')
    """
    try:
        base, rest = tzname.split('/', 1)
    except ValueError:
        base, rest = None, tzname
    rest = rest.replace('_', ' ')
    try:
        mid, rest = rest.split('/')
        if mid != base and mid != country:
            rest = '{0} ({1})'.format(rest, mid)
    except ValueError:
        pass
    return (tzname, rest)


def timezone_choices(countries, timezones, key):
    yield '', ''
    for iso3166, country in sorted(countries.items(), key=key):
        zones = timezones.get(iso3166, [])
        zones = [timezone_choice(zone, country) for zone in zones]
        zones.sort(key=key)
        yield country, zones


DAY_CHOICES = [('', '')] + list(zip(*[range(1, 32)] * 2))

MONTH_CHOICES = (
    ('', ''),
    (1, 'January'),
    (2, 'February'),
    (3, 'March'),
    (4, 'April'),
    (5, 'May'),
    (6, 'June'),
    (7, 'July'),
    (8, 'August'),
    (9, 'September'),
    (10, 'October'),
    (11, 'November'),
    (12, 'December'),
)

HOUR_CHOICES = [('', '')] + list(zip(*[range(0, 24, 1)] * 2))

MINUTES = range(0, 60)

MINUTE_CHOICES = [('', '')] + list(zip(MINUTES, ['%02d' % m for m in MINUTES]))

TIMEZONE_CHOICES = list(timezone_choices(
    pytz.country_names, pytz.country_timezones, itemgetter(1)))

# Add UTC and MGT
TIMEZONE_CHOICES.insert(1, (_('Universal'), [
    ('UTC', _("Coordinated Universal Time")),
    ('GMT', _("Greenwich Mean Time")),
]))


class SelectTimezoneForm(forms.Form):
    """
    Used by touchtechnology.common.context_processors.tz to generate a
    dropdown list of timezones to be used with the TimeZone middleware and
    set-timezone view.
    """
    timezone = forms.ChoiceField(choices=TIMEZONE_CHOICES,
                                 initial=timezone.get_current_timezone)



import datetime

import factory
import factory.fuzzy
from django.utils import timezone
from example_app import models


class TestDateTimeFieldFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.TestDateTimeField

    datetime = factory.fuzzy.FuzzyDateTime(
        datetime.datetime(2013, 7, 15, tzinfo=timezone.utc),
        datetime.datetime(2015, 6, 15, tzinfo=timezone.utc),
        force_microsecond=0,
    )


class RelativeFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Relative

    name = factory.Sequence(lambda n: 'Sample %d' % n)

    link = factory.SubFactory(TestDateTimeFieldFactory)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0002_last_modified'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='placeholder',
            options={'ordering': ('path',)},
        ),
        migrations.AlterField(
            model_name='pagecontent',
            name='label',
            field=models.SlugField(choices=[('copy', 'copy')], default='copy', max_length=20, verbose_name='CSS class'),
        ),
    ]



import logging

from modelforms.forms import ModelForm

logger = logging.getLogger(__name__)


class RedefineModelForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(RedefineModelForm, self).__init__(*args, **kwargs)
        for field, _kw in self.Meta.redefine:
            kw = {}
            for key, val in _kw.items():
                if callable(val) and not type(val) == type:
                    kw[key] = val(self.fields[field])
                else:
                    kw[key] = val
            model_field = self.Meta.model._meta.get_field(field)
            self.fields[field] = model_field.formfield(**kw)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import touchtechnology.common.db.models
import touchtechnology.common.mixins
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PythonPackage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
                ('version', models.CharField(max_length=20)),
                ('latest', models.CharField(max_length=20)),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'ordering': ('name',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SitemapNode',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('object_id', models.PositiveIntegerField(null=True, blank=True)),
                ('require_https', models.BooleanField(default=False, help_text='Force this to be served via HTTPS.', verbose_name='HTTPS Required')),
                ('enabled', models.BooleanField(default=True, help_text="Set this to 'No' to disable this object and it's children on the site.", verbose_name='Enabled')),
                ('hidden_from_navigation', models.BooleanField(default=False, help_text="When set to 'Yes' this object will still be available, but will not appear in menus.", verbose_name='Hide from menus')),
                ('hidden_from_sitemap', models.BooleanField(default=False, help_text="When set to 'Yes' this object will not be listed in the auto-generated sitemap.", verbose_name='Hide from sitemap')),
                ('hidden_from_robots', models.BooleanField(default=False, help_text="Set this to 'Yes' to prevent search engines from indexing this part of the site.<br />\n<strong>Warning:</strong> this may affect your ranking in search engines.", verbose_name='Hide from spiders')),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('content_type', models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True, on_delete=models.PROTECT)),
                ('parent', models.ForeignKey(related_name=b'children', verbose_name='Parent', blank=True, to='common.SitemapNode', null=True, on_delete=models.PROTECT)),
                ('restrict_to_groups', touchtechnology.common.db.models.ManyToManyField(help_text='If you select one or more of these groups your visitors will need to be logged in and a member of an appropriate group to view this part of the site.', to='auth.Group', verbose_name='Restrict to Groups', blank=True)),
            ],
            options={
                'ordering': ('tree_id', 'lft'),
            },
            bases=(touchtechnology.common.mixins.NodeRelationMixin, models.Model),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0023_competition__add__match_uuid_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='match',
            name='uuid',
            field=models.UUIDField(
                unique=True,
                editable=False,
                db_index=True,
            ),
        ),

        migrations.RunSQL(
            "SELECT 1;",
            "SELECT 1;",
            state_operations=[
                migrations.AlterField(
                    model_name='match',
                    name='uuid',
                    field=models.UUIDField(
                        default=uuid.uuid4,
                        unique=True,
                        editable=False,
                        db_index=True,
                    ),
                ),
            ],
        ),

    ]



import factory
from django.contrib.auth.models import User
from django.utils import timezone
from touchtechnology.common.models import SitemapNode


class SitemapNodeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SitemapNode

    title = factory.Faker('country')


class UserFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ('username',)

    username = factory.Sequence(lambda n: 'username{0}'.format(n + 1))

    first_name = factory.Faker('first_name')
    last_name = factory.Faker('last_name')

    email = factory.LazyAttribute(
        lambda a: '{0}@example.com'.format(a.first_name.lower()))
    date_joined = factory.LazyFunction(timezone.now)

    password = factory.PostGenerationMethodCall('set_password', 'password')



import datetime
import logging
import time

from dateutil.parser import parse
from django.db.models import Count
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.utils.functional import wraps
from tournamentcontrol.competition.models import Club, Competition, Season

try:
    from urllib.parse import ParseResult
except ImportError:
    from urlparse import ParseResult

logger = logging.getLogger(__name__)


@method_decorator
def registration(f, *a, **kw):
    @wraps(f)
    def _decorated(*args, **kwargs):
        club_id = kwargs.pop("club_id", None)
        person_id = kwargs.pop("person_id", None)
        season_id = kwargs.pop("season_id", None)
        team_id = kwargs.pop("team_id", None)

        if club_id:
            club = get_object_or_404(Club, pk=club_id)
            kwargs["club"] = club

            if person_id:
                person = get_object_or_404(club.members, pk=person_id)
                kwargs["person"] = person

            if season_id:
                season = get_object_or_404(
                    Season, pk=season_id, competition__clubs=club
                )
                kwargs["season"] = season

                # Knowing the season implies the competition
                kwargs["competition"] = season.competition

                # List of teams for the season.
                kwargs["teams"] = club.teams.filter(division__season=season)

            if team_id:
                team = get_object_or_404(club.teams, pk=team_id)
                kwargs["team"] = team

                # Knowing the team implies the division, season, competition.
                kwargs["division"] = team.division
                kwargs["season"] = team.division.season
                kwargs["competition"] = team.division.season.competition

        extra_context = kwargs.pop("extra_context", {})
        extra_context.update(kwargs)
        kwargs["extra_context"] = extra_context

        return f(*args, **kwargs)

    return _decorated


def competition_by_pk(f, *a, **kw):
    @wraps(f)
    def _decorated(request, *args, **kwargs):
        competition_id = kwargs.pop("competition_id", None)
        division_id = kwargs.pop("division_id", None)
        season_id = kwargs.pop("season_id", None)
        stage_id = kwargs.pop("stage_id", None)
        team_id = kwargs.pop("team_id", None)
        venue_id = kwargs.pop("venue_id", None)
        ground_id = kwargs.pop("ground_id", None)
        pool_id = kwargs.pop("pool_id", None)
        match_id = kwargs.pop("match_id", None)

        datestr = kwargs.pop("datestr", None)
        timestr = kwargs.pop("timestr", None)

        if competition_id:
            competition = get_object_or_404(Competition, pk=competition_id)
            kwargs["competition"] = competition
            if season_id:
                season = get_object_or_404(
                    competition.seasons.select_related("competition"), pk=season_id
                )
                kwargs["season"] = season
                if division_id:
                    division = get_object_or_404(
                        season.divisions.select_related("season__competition"),
                        pk=division_id,
                    )
                    kwargs["division"] = division
                    if stage_id:
                        stage = get_object_or_404(
                            division.stages.select_related(
                                "division__season__competition"
                            ),
                            pk=stage_id,
                        )
                        kwargs["stage"] = stage
                        if team_id:
                            team = get_object_or_404(
                                stage.undecided_teams.select_related(
                                    "stage__division__season__competition"
                                ),
                                pk=team_id,
                            )
                            kwargs["team"] = team

                        if pool_id:
                            pool = get_object_or_404(
                                stage.pools.select_related(
                                    "stage__division__season__competition"
                                ),
                                pk=pool_id,
                            )
                            kwargs["pool"] = pool

                        if match_id:
                            match = get_object_or_404(
                                stage.matches.select_related(
                                    "stage__division__season__competition"
                                ),
                                pk=match_id,
                            )
                            kwargs["match"] = match

                    elif team_id:
                        team = get_object_or_404(
                            division.teams.select_related(
                                "division__season__competition"
                            ),
                            pk=team_id,
                        )
                        kwargs["team"] = team
                elif venue_id:
                    venue = get_object_or_404(
                        season.venues.select_related("season__competition"), pk=venue_id
                    )
                    kwargs["venue"] = venue
                    if ground_id:
                        ground = get_object_or_404(
                            venue.grounds.select_related("venue__season__competition"),
                            pk=ground_id,
                        )
                        kwargs["ground"] = ground

                        if match_id:
                            match = get_object_or_404(ground.matches, pk=match_id)
                            kwargs["match"] = match
                    elif match_id:
                        match = get_object_or_404(venue.matches, pk=match_id)
                        kwargs["match"] = match

        if datestr:
            kwargs["date"] = parse(datestr).date()
            if timestr:
                kwargs["time"] = datetime.time(*time.strptime(timestr, "%H%M")[3:5])

        kwargs["base_url"] = ParseResult(
            "https" if request.is_secure() else "http",
            request.get_host(),
            "/",
            "",
            "",
            "",
        ).geturl()

        extra_context = kwargs.pop("extra_context", {})
        extra_context.update(kwargs)
        kwargs["extra_context"] = extra_context

        return f(request, *args, **kwargs)

    return _decorated


competition = method_decorator(competition_by_pk)


def competition_by_slug(f, *a, **kw):
    @wraps(f)
    def _decorated(*args, **kwargs):
        # deep dive into the object internals and fetch the Application
        # instance so we can call the sitemapnodes method.
        try:
            manager = f.__closure__[1].cell_contents._competitions
            # manager = f.func_closure[1].cell_contents._competitions
        # except AttributeError:
        #     manager = f.__closure__[1].cell_contents._competitions
        except TypeError:
            manager = Competition.objects

        club_slug = kwargs.pop("club", None)
        competition_slug = kwargs.pop("competition", None)
        division_slug = kwargs.pop("division", None)
        pool_slug = kwargs.pop("pool", None)
        season_slug = kwargs.pop("season", None)
        stage_slug = kwargs.pop("stage", None)
        team_slug = kwargs.pop("team", None)
        match_pk = kwargs.pop("match", None)

        datestr = kwargs.pop("datestr", None)
        timestr = kwargs.pop("timestr", None)

        base = manager.prefetch_related(
            "seasons",
            "seasons__divisions",
            "seasons__divisions__stages",
            "seasons__divisions__teams",
        )

        # see if there is an original node, we might need it later
        o_node = kwargs.pop("node", None)

        if competition_slug:
            competition = get_object_or_404(base, slug=competition_slug)
            kwargs["competition"] = competition
            if season_slug:
                season = get_object_or_404(competition.seasons, slug=season_slug)
                kwargs["season"] = season

                # List of clubs participating in this season.
                kwargs["clubs"] = competition.clubs.filter(
                    teams__division__season=season
                ).distinct()

                # We should deprecate this if it is unused in templates.
                kwargs["other_seasons"] = competition.seasons.exclude(slug=season_slug)

                # So we can build a navigation hierarchy, select extra column
                # which tells us if the season in a list of seasons is the
                # current scope.
                kwargs["seasons"] = competition.seasons.extra(
                    select={"current": "id = %s"}, select_params=(season.pk,)
                )

                # List of venues setup for this season.
                kwargs["venues"] = season.venues.all()

                if division_slug:
                    division = get_object_or_404(season.divisions, slug=division_slug)
                    kwargs["division"] = division

                    # Because Division and StageGroup use the same template
                    # we will use special context value of parent as the base
                    # for ladders and matches_by_date (both models have these
                    # methods implemented.
                    kwargs["parent"] = division

                    # So we can build a navigation hierarchy, select extra
                    # column which tells us if the division in a list of
                    # divisions is the current scope.
                    kwargs["divisions"] = season.divisions.extra(
                        select={"current": "id = %s"}, select_params=(division.pk,)
                    )

                    if stage_slug:
                        stage = get_object_or_404(
                            division.stages.annotate(pool_count=Count("pools")),
                            slug=stage_slug,
                        )
                        kwargs["stage"] = stage

                        if pool_slug:
                            pool = get_object_or_404(stage.pools, slug=pool_slug)
                            kwargs["pool"] = pool

                            # Overload the parent set above.
                            kwargs["parent"] = pool

                    if team_slug:
                        team = get_object_or_404(
                            division.teams.select_related("club"), slug=team_slug
                        )
                        kwargs["team"] = team

                        # List of players for this team.
                        players = team.people.select_related("person").filter(
                            is_player=True
                        )
                        players = players.extra(
                            select={"has_number": "number IS NULL"},
                            order_by=(
                                "has_number",
                                "number",
                                "person__last_name",
                                "person__first_name",
                            ),
                        )
                        kwargs["players"] = players

                    if match_pk:
                        match = get_object_or_404(division.matches, pk=match_pk)
                        kwargs["match"] = match

                elif match_pk:
                    match = get_object_or_404(season.matches, pk=match_pk)
                    kwargs["match"] = match

        if club_slug:
            club = get_object_or_404(Club, slug=club_slug)
            kwargs["club"] = club
            season = kwargs.get("season")
            if season:
                kwargs["teams"] = club.teams.filter(
                    division__season=season
                ).prefetch_related("division", "stage_group")

        if datestr:
            kwargs["date"] = parse(datestr).date()
            if timestr:
                kwargs["time"] = datetime.time(*time.strptime(timestr, "%H%M")[3:5])

        logger.debug("o_node = {0}".format(o_node))

        logger.debug('kwargs["node"] = {node}'.format(node=o_node))

        extra_context = kwargs.pop("extra_context", {})
        extra_context.update(kwargs)
        kwargs["extra_context"] = extra_context

        return f(*args, **kwargs)

    return _decorated


competition_slug = method_decorator(competition_by_slug)



from django.db.models import F
from tournamentcontrol.competition.signals.decorators import (
    disable_for_loaddata,
)


@disable_for_loaddata
def delete_team(sender, instance, *args, **kwargs):
    """
    When a ``Team`` is deleted we want to fix the ordering of the neighbours
    to not leave any gaps in sequences.
    """
    instance.division.teams.filter(order__gt=instance.order) \
                           .update(order=F('order') - 1)



from __future__ import unicode_literals

from django.db import models
from django.db.models import DateTimeField, ManyToManyField
from django.template.defaultfilters import slugify
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from imagekit.models import ImageSpecField
from touchtechnology.admin.mixins import AdminUrlMixin
from touchtechnology.common.db.models import BooleanField, HTMLField
from touchtechnology.news.app_settings import (
    CONTENT_LABEL_CHOICES, DETAIL_IMAGE_KWARGS, DETAIL_IMAGE_PROCESSORS, THUMBNAIL_IMAGE_KWARGS,
    THUMBNAIL_IMAGE_PROCESSORS,
)
from touchtechnology.news.image_processors import processor_factory
from touchtechnology.news.query import ArticleQuerySet, CategoryQuerySet


class AdminUrlModel(AdminUrlMixin, models.Model):

    class Meta:
        abstract = True

    def _get_admin_namespace(self):
        return 'admin:news:%s' % self._meta.model_name

    def _get_url_args(self):
        return (self.pk,)


@python_2_unicode_compatible
class Article(AdminUrlModel):

    headline = models.CharField(max_length=150, verbose_name=_('Headline'))
    slug = models.SlugField(max_length=150, verbose_name=_('Slug'))
    slug_locked = BooleanField(default=False, verbose_name=_('Locked Slug'))

    image = models.ImageField(
        blank=True, null=True, upload_to='news', verbose_name=_('Image'))

    abstract = models.TextField(verbose_name=_('Abstract'))

    published = DateTimeField(
        verbose_name=_('Published'),
        help_text=_("Set a date & time in the future to schedule "
                    "an announcement."))

    byline = models.CharField(
        max_length=75, blank=True, verbose_name=_('Byline'))
    keywords = models.CharField(
        max_length=255, blank=True, verbose_name=_('Keywords'))
    categories = ManyToManyField(
        'Category', blank=True, verbose_name=_('Categories'))

    is_active = BooleanField(default=True, verbose_name=_('Enabled'))

    # image and imagekit spec files
    thumbnail = ImageSpecField(
        source='image',
        processors=processor_factory(THUMBNAIL_IMAGE_PROCESSORS),
        **THUMBNAIL_IMAGE_KWARGS)
    detail_image = ImageSpecField(
        source='image',
        processors=processor_factory(DETAIL_IMAGE_PROCESSORS),
        **DETAIL_IMAGE_KWARGS)

    last_modified = DateTimeField(auto_now=True)

    objects = ArticleQuerySet.as_manager()

    class Meta:
        ordering = ('-published',)

    def clean(self):
        if not self.slug_locked or not self.slug:
            self.slug = slugify(self.headline)

    def __str__(self):
        return self.headline

    @property
    def copy(self):
        """
        Backwards compatibility -- concatenate all of the content sections
        into a single string and return this instead.
        """
        return ''.join(self.content.sections.values_list('copy', flat=True))


class ArticleContent(models.Model):

    article = models.ForeignKey(
        Article, related_name='content', verbose_name=_('Article'),
        on_delete=models.PROTECT)
    label = models.SlugField(
        max_length=20, choices=CONTENT_LABEL_CHOICES,
        verbose_name=_('CSS class'))
    sequence = models.PositiveIntegerField(verbose_name=_('Sequence'))
    copy = HTMLField(_("Copy"), blank=True)

    class Meta:
        ordering = ('sequence',)

    def __repr__(self):
        return '<ArticleContent: #%d, article=%d>' % (self.pk, self.article.pk)


@python_2_unicode_compatible
class Category(AdminUrlModel):

    title = models.CharField(max_length=75, verbose_name=_('Title'))
    short_title = models.CharField(
        max_length=50, blank=True, verbose_name=_('Short Title'),
        help_text=_("Used in navigation, a shorter alternative "
                    "to the main title."))
    slug = models.SlugField(max_length=75, verbose_name=_('Slug'))
    slug_locked = BooleanField(default=False, verbose_name=_('Locked Slug'))

    is_active = BooleanField(default=True, verbose_name='Enabled')

    hidden_from_navigation = BooleanField(
        default=False, verbose_name=_("Hide from menus"),
        help_text=_("When set to 'Yes' this object will still be available, "
                    "but will not appear in menus."))

    last_modified = DateTimeField(auto_now=True)

    objects = CategoryQuerySet.as_manager()

    class Meta:
        verbose_name_plural = 'categories'

    def clean(self):
        if not self.slug_locked or not self.slug:
            self.slug = slugify(self.title)

    def get_absolute_url(self):
        kwargs = {'category': self.slug}
        return reverse('news:category-index', kwargs=kwargs)

    def __str__(self):
        return self.title



from django.db import models
from tournamentcontrol.competition.signals.custom import match_forfeit  # noqa
from tournamentcontrol.competition.signals.ladders import (  # noqa
    changed_points_formula, scale_ladder_entry, team_ladder_entry_aggregation,
)
from tournamentcontrol.competition.signals.matches import (  # noqa
    match_saved_handler, notify_match_forfeit_email,
)
from tournamentcontrol.competition.signals.places import (  # noqa
    set_ground_latlng, set_ground_timezone,
)
from tournamentcontrol.competition.signals.teams import delete_team  # noqa


def delete_related(sender, instance, *args, **kwargs):
    for ro, __ in [
            (f, f.model)
            for f in instance._meta.get_fields()
            if (f.one_to_many or f.one_to_one) and
            f.auto_created and not f.concrete]:
        name = ro.get_accessor_name()
        if isinstance(ro.field, models.ManyToManyField):
            continue
        if isinstance(instance, ro.model):
            continue
        manager = getattr(instance, name)
        for obj in manager.all():
            obj.delete()



from django.conf import settings
from django.conf.urls import url
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from touchtechnology.admin.base import AdminComponent
from touchtechnology.common.decorators import superuser_login_required_m


class Settings(AdminComponent):
    verbose_name = _('Settings')
    visible = False

    def __init__(self, app, name='settings', app_name='settings'):
        super(Settings, self).__init__(app, name, app_name)

    @property
    def template_base(self):
        return 'touchtechnology/admin/settings'

    def get_urls(self):
        urlpatterns = super(Settings, self).get_urls()
        urlpatterns += [
            url(r'^site/$', self.edit_site, name='site'),
        ]
        return urlpatterns

    @superuser_login_required_m
    def index(self, request, *args, **kwargs):
        templates = self.template_path('index.html')
        return self.render(request, templates, kwargs)

    @superuser_login_required_m
    def edit_site(self, request, **extra_context):
        tenant = getattr(request, 'tenant', None)
        if tenant is not None:
            return self.generic_edit(request, tenant.__class__,
                                     instance=tenant,
                                     form_class=tenant.limited_form_class,
                                     extra_context=extra_context)
        return self.generic_edit(request, Site,
                                 pk=settings.SITE_ID,
                                 extra_context=extra_context)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-25 08:13
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0003_drop_pythonpackage'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitemapnode',
            name='kwargs',
            field=django.contrib.postgres.fields.jsonb.JSONField(default={}),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('competition', '0014_match_play_at_blank'),
    ]

    operations = [
        migrations.AddField(
            model_name='person',
            name='user',
            field=models.OneToOneField(null=True, blank=True, to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _


def boolean_coerce(value):
    if value in {1, '1'}:
        return True
    if value in {0, '0'}:
        return False


class BooleanChoiceField(forms.TypedChoiceField):
    widget = forms.RadioSelect

    def __init__(self, *args, **kwargs):
        defaults = {
            'choices': [
                ('1', _('Yes')),
                ('0', _('No')),
            ],
            'coerce': boolean_coerce,
            'required': True,
        }
        defaults.update(kwargs)
        super(BooleanChoiceField, self).__init__(*args, **defaults)

    def prepare_value(self, value):
        if value is not None:
            return str(int(value))



from django import forms
from django.conf import settings
from django.forms.models import inlineformset_factory
from django.utils.translation import ugettext_lazy as _
from modelforms.forms import ModelForm
from touchtechnology.common.forms.mixins import (
    BootstrapFormControlMixin, SuperUserSlugMixin,
)
from touchtechnology.content.forms import PlaceholderConfigurationBase
from touchtechnology.news.app_settings import NEWS_CONTENT_BLOCKS
from touchtechnology.news.models import Article, ArticleContent, Category


class ConfigurationForm(PlaceholderConfigurationBase):

    def __init__(self, *args, **kwargs):
        super(ConfigurationForm, self).__init__(*args, **kwargs)
        choices = [('', _('Please select...'))]
        choices += list(Category.objects.values_list('slug', 'title'))
        self.fields['category'] = forms.ChoiceField(
            choices=choices, required=False)
        self.fields['category'].widget.attrs.update(
            {'class': 'form-control'})


class ArticleForm(SuperUserSlugMixin, ModelForm):
    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        if not self.fields['categories'].queryset.count():
            self.fields.pop('categories', None)
        self.fields['image'].required = \
            getattr(settings, 'TOUCHTECHNOLOGY_NEWS_IMAGE_REQUIRED', True)

    class Meta:
        model = Article
        fields = (
            'headline',
            'image',
            'abstract',
            'published',
            'slug',
            'slug_locked',
            'byline',
            'keywords',
            'categories',
            'is_active',
        )


class ArticleContentForm(BootstrapFormControlMixin, ModelForm):
    def __init__(self, *args, **kwargs):
        super(ArticleContentForm, self).__init__(*args, **kwargs)
        # hide the label field if it may only contain one value
        choices = [
            each[0]
            for each in self.fields['label'].choices
            if each[0]
        ]
        if len(choices) < 2:
            self.fields['label'].widget = forms.HiddenInput()
            self.initial['label'] = choices[0]

    class Meta:
        model = ArticleContent
        fields = ('copy', 'label', 'sequence')
        widgets = {
            'sequence': forms.HiddenInput,
        }


BaseArticleContentFormset = inlineformset_factory(
    Article, ArticleContent, form=ArticleContentForm,
    can_delete=False, extra=0)


class ArticleContentFormset(BaseArticleContentFormset):
    def total_form_count(self):
        return max(self.get_queryset().count(), NEWS_CONTENT_BLOCKS)

    def _construct_form(self, i, **kwargs):
        return super(ArticleContentFormset, self)._construct_form(
            i, empty_permitted=False, initial={'sequence': i + 1})


class CategoryForm(SuperUserSlugMixin, ModelForm):
    class Meta:
        model = Category
        fields = (
            'title',
            'short_title',
            'slug',
            'slug_locked',
            'is_active',
            'hidden_from_navigation',
        )



from django.core import validators
from django.utils.translation import ugettext_lazy as _

validate_hashtag = validators.RegexValidator(
    r'^(?:#)(\w+)$',
    _("Enter a valid value. Make sure you include the # symbol."),
)



import datetime

import factory
import factory.fuzzy
from django.utils import timezone
from django.utils.text import slugify
from touchtechnology.news.models import Article, Category


class ArticleFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Article

    headline = factory.Faker('sentence')
    abstract = factory.Faker('paragraph')
    published = factory.fuzzy.FuzzyDateTime(
        datetime.datetime(2013, 7, 15, tzinfo=timezone.utc),
        force_microsecond=0)
    slug = factory.LazyAttribute(lambda a: slugify(a.headline))


class CategoryFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Category

    title = factory.Faker('country')
